@@ -58,7 +58,7 @@
 #define PROGRAM_NAME "linecut"
 
 /* The version of this program (floating-point value). */
-#define PROGRAM_VERSION 0.1
+#define PROGRAM_VERSION 0.2
 
 /* The authors of this program. */
 #define AUTHORS "Steven Schubiger"
@@ -159,12 +159,12 @@ Usage: %s [OPTION]... [-r/--range] RANGE [FILE]...\n\
 "), program_name);
       fputs (_("\
 Print selected lines of each FILE to standard output.\n\
-With more than one FILE, precede the output for each file with a header\n\
-giving the file name. With no FILE, or when FILE is -, read standard input.\n\
+With more than one FILE, precede the output of each file with a header\n\
+giving its file name. With no FILE, or when FILE is -, read standard input.\n\
 \n\
 "), stdout);
       fputs (_("\
-Mandatory arguments to long options are mandatory for short options too.\n\
+Mandatory arguments to long options are also mandatory for short options.\n\
 "), stdout);
       fputs (_("\
   -n, --number             number all output lines\n\
@@ -190,7 +190,7 @@ prefix. LINE_END may have a plus sign as prefix which denotes advancement of\n\
 lines beginning from LINE_START.\n\
 \n\
 If exactly one incomplete set is provided (with a separating colon) then certain\n\
-defaults will be assumed. If LINE_START is missing it will default to start\n\
+defaults will be assumed: If LINE_START is missing it will default to start\n\
 from first line. If LINE_END is missing it will default to end at last line.\n\
 "), stdout);
       fputs (_("\
@@ -617,23 +617,121 @@ range_file (const char *filename)
   return ok;
 }
 
+void
+parse_range_sets (char *ranges_arg)
+{
+  char *buf, *buf_orig, *range_digits, *range_iter;
+  bool insert_range_at_end, seen_range_sep;
+  int digits_count, digit_increment, ranges_seen, sets_seen;
+
+  insert_range_at_end = false;
+  seen_range_sep = false;
+
+  sets_seen = 0;
+  range_iter = ranges_arg;
+
+  while (*range_iter)
+    {
+      if (*range_iter++ == ':')
+        sets_seen++;
+    }
+  if (sets_seen == 1)
+    {
+      if (ranges_arg[0] == ':')
+        {
+          ranges[0] = 1;
+          range_max++;
+        }
+      else if (ranges_arg[strlen(ranges_arg) - 1] == ':')
+        insert_range_at_end = true;
+    }
+  ranges_seen = 0;
+  while (*ranges_arg)
+    {
+      if (VALID_RANGE (*ranges_arg))
+        {
+          digits_count = 0;
+          range_digits = ranges_arg;
+          while (VALID_RANGE (*range_digits))
+            {
+              digits_count++;
+              range_digits++;
+            }
+          digits_count++;
+          buf = xcalloc (digits_count, sizeof (char));
+          buf_orig = buf;
+          while (VALID_RANGE (*ranges_arg))
+            {
+              *buf = *ranges_arg++;
+              buf++;
+            }
+          *buf = '\0';
+          buf = buf_orig;
+
+          if (*buf == '+')
+            {
+              if ((range_max + 1) % 2 == 0)
+                {
+                  digit_increment = atoi (buf);
+                  ranges[range_max] = ranges[range_max - 1] + digit_increment;
+                }
+              else
+                error (EXIT_FAILURE, 0, _("+ prefix can not be applied to starting lines"));
+            }
+          else
+            ranges[range_max] = atol (buf);
+          range_max++;
+          free (buf);
+
+          if (range_max > MAX_RANGE_SETS)
+            error (EXIT_FAILURE, 0, _("only %d sets permitted"), MAX_RANGE_SETS / 2);
+
+          ranges_seen++;
+        }
+      else
+        {
+          if (ranges_seen == 2)
+            {
+              if (*ranges_arg != ',')
+                error (EXIT_FAILURE, 0, _("comma as set separator required"));
+              ranges_seen = 0;
+              seen_range_sep = false;
+            }
+          else if (ranges_seen == 1 && !seen_range_sep)
+            {
+              if (*ranges_arg != ':')
+                error (EXIT_FAILURE, 0, _("colon as range separator required"));
+              else
+                seen_range_sep = true;
+            }
+          else
+            {
+              if (sets_seen != 1)
+                {
+                  if (!VALID_RANGE (*ranges_arg))
+                    error (EXIT_FAILURE, 0, _("range must be number"));
+                }
+            }
+          ranges_arg++;
+        }
+     }
+  if (insert_range_at_end)
+    ranges[range_max] = -1;
+}
+
 int
 main(int argc, char **argv) 
 {
-  char *buf, *buf_orig, *ranges_arg, *range_digits, *range_iter;
+  char *ranges_arg;
   enum header_mode header_mode = multiple_files;
-  bool insert_range_at_end, ok, seen_range_sep;
-  int c, i, digits_count, digit_increment, ranges_seen, sets_seen;
+  bool ok = true;
+  int c, i;
 
   /* Initializer for file_list if no file-arguments
      were specified on the command line.  */
   static char const *const default_file_list[] = {"-", NULL};
   char const *const *file_list;
 
-  insert_range_at_end = false;
-  ok = true;
-  seen_range_sep = false;
-
   program_name = argv[0];
   setlocale (LC_ALL, "");
   bindtextdomain (PACKAGE, LOCALEDIR);
@@ -641,6 +739,9 @@ main(int argc, char **argv)
 
   atexit (close_stdout);
 
+  if (argc == 1)
+    usage (EXIT_FAILURE);
+
   while ((c = getopt_long (argc, argv, "nqr:vhV", long_options, NULL))
           != -1)
     {
@@ -677,111 +778,17 @@ main(int argc, char **argv)
         }
     }
 
-  if (argc == 1)
-    usage (EXIT_FAILURE);
-  else
-    {  
-      if (!range_mode)
+  if (!range_mode)
+    {
+      if (optind < argc)
         {
-          if (optind < argc)
-            {
-              ranges_arg = argv[optind];
-              optind++;
-            }
-          else
-            usage (EXIT_FAILURE);
+          ranges_arg = argv[optind];
+          optind++;
         }
-      sets_seen = 0;
-      range_iter = ranges_arg;
-
-      while (*range_iter)
-	{
-          if (*range_iter++ == ':')
-            sets_seen++;
-	}
-      if (sets_seen == 1)
-        {
-          if (ranges_arg[0] == ':')
-            {
-              ranges[0] = 1;
-              range_max++;
-            }
-          else if (ranges_arg[strlen(ranges_arg) - 1] == ':')
-            insert_range_at_end = true;
-	}
-      ranges_seen = 0;
-      while (*ranges_arg)
-        {
-          if (VALID_RANGE (*ranges_arg))
-            {
-              digits_count = 0;
-              range_digits = ranges_arg;
-              while (VALID_RANGE (*range_digits))
-                {
-                  digits_count++;
-                  range_digits++;
-                }
-              digits_count++;
-              buf = xcalloc (digits_count, sizeof (char));
-              buf_orig = buf;
-              while (VALID_RANGE (*ranges_arg))
-                {
-                  *buf = *ranges_arg++;
-                  buf++;
-                }
-              *buf = '\0';
-              buf = buf_orig;
-
-              if (*buf == '+')
-                {
-                  if ((range_max + 1) % 2 == 0)
-                    {
-                      digit_increment = atoi (buf);
-                      ranges[range_max] = ranges[range_max - 1] + digit_increment;
-                    }
-                  else
-                    error (EXIT_FAILURE, 0, _("+ prefix can not be applied to starting lines"));
-                }
-              else
-                ranges[range_max] = atol (buf);
-              range_max++;
-              free (buf);
-
-              if (range_max > MAX_RANGE_SETS)
-                error (EXIT_FAILURE, 0, _("only %d sets permitted"), MAX_RANGE_SETS / 2);
-
-              ranges_seen++;
-            }
-          else
-            {
-              if (ranges_seen == 2)
-                {
-                  if (*ranges_arg != ',')
-                    error (EXIT_FAILURE, 0, _("comma as set separator required"));
-                  ranges_seen = 0;
-                  seen_range_sep = false;
-                }
-              else if (ranges_seen == 1 && !seen_range_sep)
-                {
-                  if (*ranges_arg != ':')
-                    error (EXIT_FAILURE, 0, _("colon as range separator required"));
-                  else
-                    seen_range_sep = true;
-                }
-              else
-                {
-                  if (sets_seen != 1)
-                    {
-                      if (!VALID_RANGE (*ranges_arg))
-                        error (EXIT_FAILURE, 0, _("range must be number"));
-                    }
-                }
-              ranges_arg++;
-            }
-         }
-      }
-  if (insert_range_at_end)
-    ranges[range_max] = -1;
+      else
+        usage (EXIT_FAILURE);
+    }
+  parse_range_sets (ranges_arg);
 
   file_list = (optind < argc
            ? (char const *const *) &argv[optind]
